
## Description

Les calculateurs servent à déterminer le montant de points fidelité à créditer lorsqu'une stratégie est appliquée. 

### Implémentation

#### Calculator

Créer une classe implémentant l'interface `BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface`

Dans la config du plugin, ajouter un événement sur lequel appliquer une stratégie de fidelité :

```php
<?php
// src/Calculator/OrderTotalPercentageCalculator.php

declare(strict_types=1);

namespace App\Calculator;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use Sylius\Component\Core\Model\OrderInterface;
use Webmozart\Assert\Assert;

final class OrderTotalPercentageCalculator implements BonusPointsStrategyCalculatorInterface
{

    public function calculate($subject, array $configuration, int $amountToDeduct = 0): int
    {
        /** @var OrderInterface $subject */
        Assert::isInstanceOf($subject, OrderInterface::class);

        $configuration = $configuration[$subject->getChannel()->getCode()];

        $total = $subject->getTotal();

        if ($amountToDeduct < 0) {
            $total += $amountToDeduct;
        }

        return (int)round($total * $configuration['percentToCalculatePoints']);
    }

    // défini si le calculateur sera appliqué pour chaque ligne d'une commande ou pour toute la commande
    public function isPerOrderItem(): bool
    {
        return false;
    }
    
    // défini sur quels types de stratégies le calculateur sera proposé lors de la création / édition d'une stratégie
    public function getCompatibleStrategyTypes(): array
    {
        return [
            BonusPointsStrategy::TYPE_ORDER,
            BonusPointsStrategy::TYPE_EVENT
        ];
    }

    public function getType(): string
    {
        return 'order_total_percentage';
    }
}
```

**Il n'y a pas de typage fort sur `$subject`, il est donc important de vérifier le type de l'objet sur lequel on fait le calcul **

#### FormTypes

Implémenter un ConfigurationType pour votre calculateur 

```php
<?php

declare(strict_types=1);

namespace App\Form\Type\Calculator;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PercentType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\Type;

final class OrderTotalPercentageConfigurationType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('percentToCalculatePoints', PercentType::class, [
                'label' => 'bitbag_sylius_bonus_points.ui.percent_to_calculate_points',
                'constraints' => [
                    new NotBlank(['groups' => ['bitbag_sylius_bonus_points']]),
                    new Type(['type' => 'integer', 'groups' => ['sylius']]),
                ],
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver
            ->setDefaults([
                'data_class' => null,
            ])
            ->setRequired('currency')
            ->setAllowedTypes('currency', 'string')
        ;
    }
}
```

Dans `buildForm` vous pouvez déclarer tous les champs dont vous avez besoin pour assurer le bon fonctionnement du calculateur.

Ces informations sont serializées en base de données dans un champs JSON et sont transmises au calculateur sous forme d'array (argument `$configuration` de `BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface::calculate`)

Les calculateurs de base du plugin sont équipés de formulaires adaptés au multi-canal (ChannelBasedConfigurationType).

```php
<?php

declare(strict_types=1);

namespace App\Form\Type\Calculator;

use Sylius\Bundle\CoreBundle\Form\Type\ChannelCollectionType;
use Sylius\Component\Core\Model\ChannelInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class ChannelBasedOrderTotalPercentageConfigurationType extends AbstractType
{
    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'entry_type' => OrderTotalPercentageConfigurationType::class,
            'entry_options' => function (ChannelInterface $channel): array {
                return [
                    'label' => $channel->getName(),
                    'currency' => $channel->getBaseCurrency()->getCode(),
                ];
            },
        ]);
    }

    public function getParent(): string
    {
        return ChannelCollectionType::class;
    }
}
```

#### Configuration

Configurer le configurateur et son formulaire comme services : 

```yaml
{# config/services/bitbag_sylius_bonus_points_plugin.yaml #}

services:
    app.form.type.channel_based_bonus_points_strategy_calculator.channel_based_order_total_percentage:
    class: App\Form\Type\Calculator\ChannelBasedOrderTotalPercentageConfigurationType
    tags: 
      - { name: 'form.type'}

    app.bonus_points_strategy_calculator.order_total_percentage:
      class: App\Calculator\OrderTotalPercentageCalculator
      tags:
        - {
          name: 'bitbag.bonus_points_strategy_calculator',
          calculator: 'order_total_percentage',
          form-type: 'App\Form\Type\Calculator\ChannelBasedOrderTotalPercentageConfigurationType',
          label: 'app.ui.order_total_percentage'
        }
```

Grâce au tag `bitbag.bonus_points_strategy_calculator` on indique au plugin d'enregistrer un calculateur dont le type est `order_total_percentage` utilisant le formulaire `ChannelBasedOrderTotalPercentageConfigurationType` dans l'Admin et représenté par le label `app.ui.order_total_percentage` dans la liste déroulante de l'admin. 

Le calculateur est désormais disponible lors de la création d'une stratégie de fidelité compatible. 
