<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Assigner;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Resolver\CartBonusPointsAmountResolver;
use DateTime;
use Doctrine\ORM\EntityManagerInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Webmozart\Assert\Assert;

final class BonusPointsAssigner implements BonusPointsAssignerInterface
{
    private CartBonusPointsAmountResolver $cartBonusPointsAmountResolver;

    private FactoryInterface $bonusPointsFactory;

    private EntityManagerInterface $bonusPointsEntityManager;

    private RepositoryInterface $customerBonusPointsRepository;

    private FactoryInterface $customerBonusPointsFactory;

    private RepositoryInterface $customerBonusPointsStrategyRepository;

    private FactoryInterface $customerBonusPointsStrategyFactory;

    private EntityManagerInterface $customerBonusPointsStrategyEntityManager;

    public function __construct(
        CartBonusPointsAmountResolver $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory,
        RepositoryInterface $customerBonusPointsStrategyRepository,
        FactoryInterface $customerBonusPointsStrategyFactory,
        EntityManagerInterface $customerBonusPointsStrategyEntityManager
    ) {
        $this->cartBonusPointsAmountResolver = $cartBonusPointsAmountResolver;
        $this->bonusPointsFactory = $bonusPointsFactory;
        $this->bonusPointsEntityManager = $bonusPointsEntityManager;
        $this->customerBonusPointsRepository = $customerBonusPointsRepository;
        $this->customerBonusPointsFactory = $customerBonusPointsFactory;
        $this->customerBonusPointsStrategyRepository = $customerBonusPointsStrategyRepository;
        $this->customerBonusPointsStrategyFactory = $customerBonusPointsStrategyFactory;
        $this->customerBonusPointsStrategyEntityManager = $customerBonusPointsStrategyEntityManager;
    }

    public function assign(OrderInterface $order): void
    {
        /** @var CustomerInterface $customer */
        $customer = $order->getCustomer();
        Assert::isInstanceOf($customer, CustomerInterface::class);

        $eligibleBonusPointsStrategies = $this->cartBonusPointsAmountResolver->getEligibleBonusPointsStrategies($order);

        $bonusPointsTotal = $this->cartBonusPointsAmountResolver->sumEligibleBonusPointsStrategiesTotal($eligibleBonusPointsStrategies);

        if ($bonusPointsTotal === 0) {
            return;
        }

        /** @var BonusPointsInterface $bonusPoints */
        $bonusPoints = $this->bonusPointsFactory->createNew();

        $bonusPoints->setOrder($order);
        $bonusPoints->setPoints($bonusPointsTotal);
        $bonusPoints->setIsUsed(false);
        $bonusPoints->setExpiresAt((new DateTime())->modify('+1 year'));

        $this->getCustomerBonusPoints($customer)->addBonusPoints($bonusPoints);

        $this->bonusPointsEntityManager->persist($bonusPoints);
        $this->bonusPointsEntityManager->flush();

        foreach ($eligibleBonusPointsStrategies as $eligibleBonusPointsStrategy) {
            $bonusPointsStrategy = $eligibleBonusPointsStrategy->getBonusPointsStrategy();
            $this->addCustomerBonusPointsStrategy($customer, $bonusPointsStrategy);
        }
    }

    private function getCustomerBonusPoints(CustomerInterface $customer): CustomerBonusPointsInterface
    {
        /** @var CustomerBonusPointsInterface $customerBonusPoints */
        $customerBonusPoints = $this->customerBonusPointsRepository->findOneBy([
                'customer' => $customer,
            ]) ?? $this->customerBonusPointsFactory->createNew();

        if (null === $customerBonusPoints->getId()) {
            $customerBonusPoints->setCustomer($customer);

            $this->customerBonusPointsRepository->add($customerBonusPoints);
        }

        return $customerBonusPoints;
    }

    private function addCustomerBonusPointsStrategy(CustomerInterface $customer, BonusPointsStrategyInterface $bonusPointsStrategy): void
    {
        if (!($customerBonusPointsStrategy = $this->customerBonusPointsStrategyRepository->findOneBy([
            'customer' => $customer,
            'bonusPointsStrategy' => $bonusPointsStrategy,
        ]))) {
            /** @var CustomerBonusPointsStrategyInterface $customerBonusPointsStrategy */
            $customerBonusPointsStrategy = $this->customerBonusPointsStrategyFactory->createNew();
            $customerBonusPointsStrategy->setCustomer($customer);
            $customerBonusPointsStrategy->setBonusPointsStrategy($bonusPointsStrategy);

            $this->customerBonusPointsStrategyEntityManager->persist($customerBonusPointsStrategy);
            $this->customerBonusPointsStrategyEntityManager->flush();
        }
    }
}
