<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Resolver;

use BitBag\SyliusBonusPointsPlugin\Calculator\DelegatingBonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Checker\Eligibility\BonusPointsStrategyEligibilityCheckerInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\AdjustmentInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\EligibleBonusPointsStrategy;
use BitBag\SyliusBonusPointsPlugin\Entity\EligibleBonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Repository\BonusPointsStrategyRepositoryInterface;
use Sylius\Component\Core\Distributor\ProportionalIntegerDistributorInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;

final class CartBonusPointsAmountResolver implements CartBonusPointsAmountResolverInterface
{
    private $bonusPointsStrategyEligibilityChecker;

    private $delegatingBonusPointsStrategyCalculator;

    private $bonusPointsStrategyRepository;

    private $proportionalIntegerDistributor;

    public function __construct(
        BonusPointsStrategyEligibilityCheckerInterface $bonusPointsStrategyEligibilityChecker,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        ProportionalIntegerDistributorInterface $proportionalIntegerDistributor
    ) {
        $this->bonusPointsStrategyEligibilityChecker = $bonusPointsStrategyEligibilityChecker;
        $this->delegatingBonusPointsStrategyCalculator = $delegatingBonusPointsStrategyCalculator;
        $this->bonusPointsStrategyRepository = $bonusPointsStrategyRepository;
        $this->proportionalIntegerDistributor = $proportionalIntegerDistributor;
    }

    public function resolveBonusPoints(OrderInterface $order): int
    {
        $eligibleBonusPointsStrategies = $this->getEligibleBonusPointsStrategies($order);

        return $this->sumEligibleBonusPointsStrategiesTotal($eligibleBonusPointsStrategies);
    }

    private function getProportionalIntegerToDeduct(BonusPointsStrategyInterface $bonusPointsStrategy, array $orderItems, int $bonusPointsAmount): array
    {
        /** @var OrderItemInterface[] $itemsEligible */
        $itemsEligible = [];

        foreach ($orderItems as $orderItem) {
            if ($this->bonusPointsStrategyEligibilityChecker->isEligible($orderItem, $bonusPointsStrategy)) {
                $itemsEligible[] = $orderItem;
            }
        }

        $totals = [];

        foreach ($itemsEligible as $item) {
            $totals[] = $item->getTotal();
        }

        $splitAmount = $this->proportionalIntegerDistributor->distribute($totals, $bonusPointsAmount);

        $splitAmountOrderItems = [];

        $i = 0;

        foreach ($itemsEligible as $item) {
            $splitAmountOrderItems[$item->getId()] = $splitAmount[$i++];
        }

        return $splitAmountOrderItems;
    }

    public function getEligibleBonusPointsStrategies(OrderInterface $order): array
    {
        $bonusPointsStrategies = $this->bonusPointsStrategyRepository->findAllActive();

        $result = [];

        /** @var BonusPointsStrategyInterface $bonusPointsStrategy */
        foreach ($bonusPointsStrategies as $bonusPointsStrategy) {
            if ($this->delegatingBonusPointsStrategyCalculator->isPerOrderItem($bonusPointsStrategy)) {
                $splitAmountOrderItems = [];
                if ($bonusPointsStrategy->isDeductBonusPoints()) {
                    $splitAmountOrderItems = $this->getProportionalIntegerToDeduct(
                        $bonusPointsStrategy,
                        $order->getItems()->toArray(),
                        $order->getAdjustmentsTotal(AdjustmentInterface::ORDER_BONUS_POINTS_ADJUSTMENT)
                    );
                }

                /** @var OrderItemInterface $orderItem */
                foreach ($order->getItems() as $orderItem) {
                    if ($this->bonusPointsStrategyEligibilityChecker->isEligible($orderItem, $bonusPointsStrategy)) {
                        $result[] = new EligibleBonusPointsStrategy(
                            $bonusPointsStrategy,
                            $this->delegatingBonusPointsStrategyCalculator->calculate(
                                $orderItem,
                                $bonusPointsStrategy,
                                $bonusPointsStrategy->isDeductBonusPoints() ? $splitAmountOrderItems[$orderItem->getId()] : 0
                            )
                        );
                    }
                }
            } else {
                $status = true;

                /** @var OrderItemInterface $orderItem */
                foreach ($order->getItems() as $orderItem) {
                    if (!$this->bonusPointsStrategyEligibilityChecker->isEligible($orderItem, $bonusPointsStrategy)) {
                        $status = false;
                    }
                }

                if (true === $status) {
                    $result[] = new EligibleBonusPointsStrategy(
                        $bonusPointsStrategy,
                        $this->delegatingBonusPointsStrategyCalculator->calculate(
                            $order,
                            $bonusPointsStrategy,
                            0
                        )
                    );
                }
            }
        }

        return $result;
    }

    /**
     * @param EligibleBonusPointsStrategyInterface[]|array $eligibleBonusPointsStrategies
     */
    public function sumEligibleBonusPointsStrategiesTotal($eligibleBonusPointsStrategies): int
    {
        $bonusPointsTotal = 0;

        foreach ($eligibleBonusPointsStrategies as $eligibleBonusPointsStrategy) {
            $bonusPointsTotal += $eligibleBonusPointsStrategy->getBonusPointsAmount();
        }

        return $bonusPointsTotal;
    }
}
